use strict;
use warnings;
package HTML::MasonX::Free 0.007;

# ABSTRACT: a secret society of HTML::Mason-using throwback modules

#pod =head1 OVERVIEW
#pod
#pod These modules, together, help make HTML::Mason work more like RJBS thinks it
#pod should have always worked.  More documentation about the general ideas will be
#pod here later, when RJBS is not pressed for time.
#pod
#pod =cut

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

HTML::MasonX::Free - a secret society of HTML::Mason-using throwback modules

=head1 VERSION

version 0.007

=head1 OVERVIEW

These modules, together, help make HTML::Mason work more like RJBS thinks it
should have always worked.  More documentation about the general ideas will be
here later, when RJBS is not pressed for time.

=head1 PERL VERSION

This library should run on perls released even a long time ago.  It should work
on any version of perl released in the last five years.

Although it may work on older versions of perl, no guarantee is made that the
minimum required version will not be increased.  The version may be increased
for any reason, and there is no promise that patches will be accepted to lower
the minimum required perl.

=head1 AUTHOR

Ricardo Signes <cpan@semiotic.systems>

=head1 CONTRIBUTOR

=for stopwords Ricardo Signes

Ricardo Signes <rjbs@semiotic.systems>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2022 by Ricardo Signes.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
